/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir-writecache.cc
 *  
 *  Description:
 *    Implementation of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "maildir.h"

#include "../../io/io.h"
#include "../../parsers/storage/storage.h"

using namespace ::std;

//------------------------------------------------------------------------
void Binc::Maildir::writeCache(void) const
{
  IO &logger = IOFactory::getInstance().get(2);

  if (readonly) return;

  const string cachefilename = path + "/bincimap-cache";
  const string uidvalfilename = path + "/bincimap-uidvalidity";

  Storage cache;
  int n = 0;
  
  Mailbox::iterator e = end();
  for (Mailbox::iterator i = begin(); i != e; ++i) {
    Message & message = *i;
    string nstr = toString(n++);
    
    cache.setSection(nstr);
    cache.insert("_UID", toString(message.getUID()));
    if (message.getSize() != -1)
	cache.insert("_Size", toString(message.getSize()));
    cache.insert("_ID", message.getID());
  }

  cache.setSection("depot");
  cache.insert("_version", CACHEFILEVERSION);

  try {
    cache.setSaveMode(Storage::STORAGE_PRETTIFY);
    cache.save(cachefilename);
  } catch (StorageException &e) {
    logger << "warning in writeCache, while saving to " << cachefilename 
	   << ": " << e.getReason() << endl;
    throw e;
  }
}
