/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/bincimapd-create.cc
 *  
 *  Description:
 *    Implementation of the CREATE command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

#include "depot.h"
#include "exceptions.h"
#include "io/io.h"
#include "mailbox/mailbox.h"
#include "operators.h"
#include "parsers/imap/imapparser.h"
#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "session.h"
#include "util/convert.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
CreateOperator::CreateOperator(void)
{
}

//----------------------------------------------------------------------
CreateOperator::~CreateOperator(void)
{
}

//----------------------------------------------------------------------
const string CreateOperator::getName(void) const
{
  return "CREATE";
}

//----------------------------------------------------------------------
int CreateOperator::getState(void) const
{
  return Session::AUTHENTICATED | Session::SELECTED;
}

//------------------------------------------------------------------------
void CreateOperator::process(Depot &depot,
			     Command &command) const
{
  try {
    depot.createMailbox(command.getMailbox());
  } catch (StorageException &e) {
    throw OperatorException(Operator::NO, e.getReason());
  }
}

//----------------------------------------------------------------------
bool CreateOperator::parse(Command &c_in) const
{
  string mailbox;

  try {
    if (!expectThisString("CREATE"))
      return false;

    if (!expectSPACE())
      throw SyntaxException("expected SPACE");

    if (!expectMailbox(mailbox))
      throw SyntaxException("expected mailbox");    
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in CreateOperator::parse");
    throw e;
  } catch (SyntaxException &e) {
    e.addReason("in CreateOperator::parse");
  }

  c_in.setName("CREATE");
  c_in.setMailbox(mailbox);

  return true;
}
