/* -*- Mode: lex; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/storage/storageparser.lex
 *  
 *  Description:
 *    Definition of the Storage lexemes, in lex
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
%{
#include <string>
#include <string.h>
#include <zlib.h>

//int my_yyinput(char *buf, int max_size);
//#ifdef YY_INPUT
//#undef YY_INPUT
//#endif
//#define YY_INPUT(b, r, ms) (r = my_yyinput(b, ms))

#include "y.tab.h"

char storage_buffer[8192];

int storage_charnr = 0;
int storage_linenr = 1;
std::string storage_quoted;

extern std::string storage_erradd;

%}

%option prefix="storageparser"

%x comment
%x quoted

%%

\<[a-z0-9A-Z]+\>	        {
				  // hex encoded string, starts with a
				  // '<', then comes hex code, ends with
				  // '>'
				  storage_charnr += strlen(storageparsertext);
				  return HEX;
                                }

[a-zA-Z0-9\-\_\^\.]+	        {
				  // a label
				  storage_charnr += strlen(storageparsertext);
				  return LABELCHARS;
				}

[ \r\t]+                        {
				  // whitespace
				  storage_charnr += strlen(storageparsertext);
				}

\/\/.*				{ 
				  // single line comment
				  storage_charnr += strlen(storageparsertext);
				}

\n				{
				  // any character
				  storage_charnr = 0;
				  storage_linenr++;
				}

\"				{
				  // begin quoted string
				  storage_quoted = "";
				  storage_charnr++;
				  BEGIN(quoted);
				}

.				{
                                  // quoted string content
				  storage_charnr++;
				  return storageparsertext[0];
				}

<quoted>\\.			{
                                  // quoted string content
				  storage_charnr += 2;
				  storage_quoted += storageparsertext[1];
				}

<quoted>\"			{
				  // end quoted string
				  storage_charnr++;
				  BEGIN(INITIAL);
				  return QUOTED;
				}

<quoted>.			{
                                  // quoted string content
				  storage_charnr++;
				  storage_quoted += storageparsertext[0];
				}

<quoted>\n			{
                                  // quoted string content
				  storage_charnr++;
				  storage_quoted += storageparsertext[0];
				}

\/\*				{
				  // begin comment
				  storage_charnr += 2;
				  BEGIN(comment); 
				}

<comment>\*\/			{ 
				  // end comment
				  storage_charnr += 2;
				  BEGIN(INITIAL); 
				}

<comment>\n			{
				  // comment content
				  storage_charnr = 0;
				  storage_linenr++;
				}

<comment>.			{
				  // comment content
				  storage_charnr++;
				}

%%

int yywrap(void)
{
  return 1;
}

int my_yyinput(char *buf, int max_size)
{
  //  int n = gzread(yyin, storage_buffer, sizeof(storage_buffer));
  int n = fread(storage_buffer, 1, sizeof(storage_buffer), yyin);
  memcpy(buf, storage_buffer, n);
  return n;
}

int yyerror(char *c)
{
  std::string storage_error;
  char buf[1024];
  snprintf(buf, sizeof(buf), "%s: %s at line %i, character %i",
         c, storage_erradd.c_str(), storage_linenr, storage_charnr);

  storage_error = buf;
  throw StorageException(storage_error);
}
